"""Module generator related to pooling operations.

- Author: Jongkuk Lim
- Contact: lim.jeikei@gmail.com
"""

from torch import nn


class GlobalAvgPool(nn.AdaptiveAvgPool2d):
    """Global average pooling module.

    Arguments: []
    """

    def __init__(self):
        """Initialize."""
        super().__init__(output_size=1)



"""Module Description.

- Author: Jongkuk Lim
- Contact: lim.jeikei@gmail.com
"""

from kindle import generator, modules
from kindle.model import Model
from kindle.trainer import TorchTrainer

__all__ = ["modules", "generator", "Model", "TorchTrainer"]



"""Module Description.

- Author: Jongkuk Lim
- Contact: lim.jeikei@gmail.com
"""

import setuptools
from setuptools import setup

setup(
    name="kindle",
    version="0.1.0",
    packages=setuptools.find_packages(),
    description="PyTorch no-code model builder.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    license=open("LICENSE").read().replace("\n", " "),
    author="Jongkuk Lim",
    author_email="lim.jeikei@gmail.com",
    url="https://github.com/JeiKeiLim/kindle",
    download_url="",
    install_requires=["tqdm>=4.56.0", "PyYAML>=5.3.1"],
    classifiers=["Programming Language :: Python :: 3.8"],
)



"""Concat module.

- Author: Jongkuk Lim
- Contact: lim.jeikei@gmail.com
"""
# pylint: disable=useless-super-delegation

from typing import List, Tuple, Union

import torch
from torch import nn


class Concat(nn.Module):
    """Concatenation module.

    Arguments: [dimension]
    """

    def __init__(self, dimension: int = 1) -> None:
        """Concatenation module.

        Args:
            dimension: concatenation axis.
        """
        super().__init__()
        self.dimension = dimension

    def forward(self, x: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]):
        """Forward."""
        return torch.cat(x, self.dimension)
